from django.db import models
from email.policy import default
import uuid
from django.db import models
from django.dispatch import receiver
from django.db.models.signals import post_save
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin 
from django.contrib.auth.models import Group  

from django.contrib.auth.base_user import BaseUserManager

class MyAccountManager(BaseUserManager):
    def create_user(self, email, first_name, phone_number,  password=None):
        if not email:
            raise ValueError('Users must have an email address')
        if not first_name:
            raise ValueError('Users must have a first name') 

        user = self.model(
            email=self.normalize_email(email),
            phone_number=phone_number,
            first_name=first_name
        )

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, first_name, phone_number,  password):
        user = self.create_user(
            email=self.normalize_email(email),
            password=password,
            phone_number=phone_number,
            first_name=first_name
        )
        user.is_admin = True
        user.is_staff = True
        user.is_superuser = True
        user.save(using=self._db)
        return user 
 
GENDER_CHOICES = (
        ('Male', 'Male'),
        ('Female', 'Female'),
        ('Other', 'Other'),
    ) 

EMPLOYMENT_TYPE_CHOICES = (
        ('Contract', 'Contract'),
        ('Permanent', 'Permanent'),
        ('Casual', 'Casual'),
    ) 

STATUS_CHOICES = (
    ('Active', 'Active'),
    ('Inactive', 'Inactive'),
    ('Blocked', 'Blocked'),
) 
MARITAL_STATUS = (
    ('MARIED', 'MARIED'),
    ('SINGLE', 'SINGLE'),
)

class Account(AbstractBaseUser, PermissionsMixin): 
    # Personal Information
    first_name = models.CharField(max_length=255, null=True, blank=True)
    second_name = models.CharField(max_length=255, null=True, blank=True)
    email = models.CharField(max_length=255, null=True, blank=True, unique=True)
    phone_number = models.CharField(max_length=255, null=True, blank=True, unique=True)
    gender = models.CharField(max_length=10, choices=GENDER_CHOICES, default='Male')
    dob = models.DateTimeField(null=True, blank=True)
    age = models.IntegerField(default=0)
    marital_status = models.CharField(max_length=8, choices=MARITAL_STATUS, default='Active')

    # Contact Information
    location = models.CharField(max_length=255, null=True, blank=True)
    country = models.CharField(max_length=255, null=True, blank=True)
    #branch = models.ForeignKey(CompanyBranch, on_delete=models.CASCADE, blank=True, null=True)

    # Employment Details
    employment_type = models.CharField(max_length=10, choices=EMPLOYMENT_TYPE_CHOICES, default='Casual')
    kra_pin = models.CharField(max_length=20, null=True, blank=True)
    status = models.CharField(max_length=8, choices=STATUS_CHOICES, default='Active')
    education = models.CharField(max_length=255, null=True, blank=True)
    nationality = models.CharField(max_length=50, null=True, blank=True)

    # Next of Kin and Emergency Contact
    next_of_kin_name = models.CharField(max_length=255, null=True, blank=True)
    next_of_kin_phone = models.CharField(max_length=20, null=True, blank=True)
    emergency_contact = models.CharField(max_length=20, null=True, blank=True)
    emergency_contact_name = models.CharField(max_length=255, null=True, blank=True)
    emergency_contact_phone = models.CharField(max_length=20, null=True, blank=True)
    emergency_contact_relationship = models.CharField(max_length=100, null=True, blank=True)
    secondary_emergency_contact_name = models.CharField(max_length=255, null=True, blank=True)
    secondary_emergency_contact_phone = models.CharField(max_length=20, null=True, blank=True)
    secondary_emergency_contact_relationship = models.CharField(max_length=100, null=True, blank=True)

    # Other Details
    average_rating = models.FloatField(default=0)
    profile_image = models.ImageField(upload_to='user_profiles/', null=True, blank=True)
    documents = models.FileField(upload_to='user_documents/', null=True, blank=True)

    # Permissions and Group 
    is_staff = models.BooleanField(default=False)
    is_admin_user = models.BooleanField(default=False)
    is_super_user = models.BooleanField(default=False)
    is_sales = models.BooleanField(default=False)
    is_purchases = models.BooleanField(default=False)
    is_profiling = models.BooleanField(default=False)
    is_production = models.BooleanField(default=False)
    is_inventory = models.BooleanField(default=False)
    is_pos = models.BooleanField(default=False)
    is_accounting = models.BooleanField(default=False)
    key = models.CharField(max_length=255, null=True, blank=True, unique=True)

    # Group assignment
    groups = models.ManyToManyField(Group, related_name='users', blank=True)

    # Timestamps
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['first_name', 'phone_number']

    objects = MyAccountManager()

    class Meta:
        db_table = "tbl_users"

    # def __str__(self):
    #     return self.id 

    def get_roles(self):
        roles = []
        if self.is_staff:
            roles.append('Staff')
        if self.is_sales:
            roles.append('Sales')
        if self.is_profiling:
            roles.append('Profiling')
        if self.is_accounting:
            roles.append('Accounting')
        if self.is_production:
            roles.append('Production')
        if self.is_inventory:
            roles.append('Inventory')
        if self.is_pos:
            roles.append('POS')
        return ', '.join(roles)

    # def save(self, *args, **kwargs):
        # Save the instance to get an ID before accessing the many-to-many relationship
        super().save(*args, **kwargs)

        # Create groups if they don't exist and assign the user to respective groups based on roles
        groups = {
            'SALES': self.is_sales,
            'PROFILING': self.is_profiling,
            'ACCOUNTING': self.is_accounting,
            'PRODUCTION': self.is_production,
            'INVENTORY': self.is_inventory,
            'POS': self.is_pos 
        } 
        # Check if the user is staff, admin user, or super user
        is_staff_admin_super = self.is_staff or self.is_admin_user or self.is_super_user

        # Add user to all roles if is_staff, is_admin_user, or is_super_user is True
        if is_staff_admin_super:
            self.is_staff = True
            self.is_admin_user = True
            self.is_super_user = True
            self.is_sales = True
            self.is_purchases = True
            self.is_profiling = True
            self.is_production = True
            self.is_inventory = True
            self.is_pos = True
            self.is_accounting = True
            super().save(*args, **kwargs)
            for group_name, _ in groups.items():
                group, created = Group.objects.get_or_create(name=group_name)
                self.groups.add(group)

        # Otherwise, assign roles based on individual flags
        else:
            for group_name, role_status in groups.items():
                group, created = Group.objects.get_or_create(name=group_name)
                if role_status:
                    self.groups.add(group)
                elif group in self.groups.all():
                    self.groups.remove(group)